﻿
using IsolationFrameworkDemo.Entity;
using IsolationFrameworkDemo.Services;
using NUnit.Framework;
using Rhino.Mocks;

namespace IsolationFrameworkDemo.Tests
{
    [TestFixture]
    public class TestHarness
    {

        [Test]
        public void When_All_Information_is_valid_An_Email_Is_Sent()
        {
            //AAA
            //Arrange
            var userName = "Fred";
            var userRepository = MockRepository.GenerateStub<IUserRepository>();
            var challangeValidator = MockRepository.GenerateStub<IChallangeValidator>();
            var emailGateway = MockRepository.GenerateMock<IEmailGateway>();
            var challengeFromDatabase = new ChallengeResponse();
            var providedChallenge = new ChallengeResponse();

            userRepository.Stub(x => x.FetchUserInformation(userName)).Return(new User());
            userRepository.Stub(x => x.GetChallangeResponseInformation(Arg<User>.Is.Anything)).Return(challengeFromDatabase);
            challangeValidator.Stub(x => x.ValidateUserInformation(challengeFromDatabase, providedChallenge))
                .Return(true);

            //emailGateway.Expect(x => x.SendEmail( Arg<string>.Is.Anything,
            //    Arg<string>.Matches(y => y.StartsWith("Login Reminder For")), 
            //    Arg<string>.Is.Anything));

            //Act 

            var userLoginService = new UserLoginService(userRepository, challangeValidator, emailGateway);

            userLoginService.EmailLoginInformation(providedChallenge, userName);

            // Assert
            emailGateway.AssertWasCalled( x => x.SendEmail( Arg<string>.Is.Anything,
                Arg<string>.Matches(y => y.StartsWith("Login Reminder For")), 
                Arg<string>.Is.Anything) );
        }
    }
}
